// Fichier model.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   			de calcul de la position du robot		*
 *                                                      *
 * .Date de cration : 23/01/08                         *
 * .Auteurs : P. Lucidarme					            *       
 *                                                      *
 * .Remarques : 										*
 *                                                      *
 *                                                      *
 ********************************************************/

#include <math.h>

#include "model.h"
#include "codeur.h"


struct 				position 	Pos_Robot;				// Position/orientation courante du robot 

unsigned int 		Codeur_val_G_vieux;					// Ancienne valeur du codeur gauche
unsigned int 		Codeur_val_D_vieux;					// Ancienne valeur du codeur droit






/*-------------------------------------------------------------------
 * Initialise le model gomtrique du robot
 * Reoit en paramtre la position/orientation de dpart du robot
*/
void Model_Init(float X, float Y, float Psi)
{
	Codeur_Lire_val( &Codeur_val_G_vieux , &Codeur_val_D_vieux);	// Lit les codeurs pour initaliser les valeurs  n-1
	Pos_Robot.X=X;
	Pos_Robot.Y=Y;
	Pos_Robot.Psi=Psi;	
}	


/*-------------------------------------------------------------------
 * Lit les codeurs et met  jour le modle gomtrique
 * Aucun paramtre
 * Dure d'execution minimum : 500s
 * Dure d'xecution maximum : 2.5ms
*/
void Model_Cinematique (void)
{
	unsigned int 		Codeur_val_G;											// Valeur courante du codeur gauche
	unsigned int 		Codeur_val_D;											// Valeur courante du codeur droit
	int			 		Vitesse_Mesuree_G; 										// Vitesse sur la roue gauche
	int			 		Vitesse_Mesuree_D; 										// Vitesse sur la roue droite
	
	Codeur_Lire_val( &Codeur_val_G,&Codeur_val_D);								// Lit les codeurs et sauvegarde dans les variables	
		
	Vitesse_Mesuree_G= Codeur_val_G - Codeur_val_G_vieux;						// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Gauche
	Vitesse_Mesuree_D= Codeur_val_D_vieux - Codeur_val_D; 						// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Droit
	
	Codeur_val_G_vieux=Codeur_val_G;											// Sauvegarde de la valeur du codeur Gauche actuel pour la prochaine IT 
	Codeur_val_D_vieux=Codeur_val_D;											// Sauvegarde de la valeur du codeur Droit actuel pour la prochaine IT 
	
	Pos_Robot.X+=K1*(Vitesse_Mesuree_D+Vitesse_Mesuree_G)*cos(Pos_Robot.Psi);	// Position du robot en X
	Pos_Robot.Y+=K1*(Vitesse_Mesuree_D+Vitesse_Mesuree_G)*sin(Pos_Robot.Psi);	// Position du robot en Y
	Pos_Robot.Psi +=K2*(Vitesse_Mesuree_D-Vitesse_Mesuree_G); 					// Calcul du differentiel de l'angle du robot
 		
	if(Pos_Robot.Psi>PI)	Pos_Robot.Psi-=2.0*PI;								// Garde PSI entre -PI et PI
	if(Pos_Robot.Psi<=-PI)	Pos_Robot.Psi+=2.0*PI;
}	



/*-------------------------------------------------------------------
 * Calcul l'angle entre le robot et la cible
 * Les paramtres passs sont XCible et YCible
 * Si le robot est confondu avec la cible, la fonction retourne 0
*/
float Angle_Robot_Cible(float XCible, float YCible)
{
	float dx,dy;															// Distance en x et en y entre le robot et la cible		
	float Teta=0;															// Angle entre le robot et la cible
	
	dx=XCible-Pos_Robot.X;													// Calcul l'cart en x entre le robot et le point
	dy=YCible-Pos_Robot.Y;													// Calcul l'cart en y entre le robot et le point				
				
	if(dx!=0 || dy!=0) 														// On vrifie qu'on est pas pile poil sur le point (sinon erreur atan2)
	{												
	//	Teta=Trigo_atan2(dy,dx)-Pos_Robot.Psi;						// Calcul de l'angle entre le robot et le point traqu
		Teta=atan2(dy,dx)-Pos_Robot.Psi;									// Calcul de l'angle entre le robot et le point traqu	
		while (Teta>PI)  Teta-=2.0*PI;										// On garde l'angle entre PI et -PI
		while (Teta<=-PI) Teta+=2.0*PI;		
	}
	
	return Teta;															// On retourn l'angle entre -PI et +PI
}	

/*-------------------------------------------------------------------
 * Retourne le carr de la distance entre le robot et la cible
 * Les paramtres passs sont XCible et YCible
 */
float Distance_Cible(float XCible, float YCible)
{
	float dx,dy;															// Distance en x et en y entre le robot et la cible		
		
	dx=XCible-Pos_Robot.X;													// Calcul l'cart en x entre le robot et le point
	dy=YCible-Pos_Robot.Y;
	return (dx*dx+dy*dy);
}	