// Fichier Moteur.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   		de la commande Moteur de propulsion			*
 *                                                      *
 * .Date de cration : 21/01/08                         *
 * .Auteurs : Came Mikael Et Guibert David              *       
 *                                                      *
 * .Descriptions :                                      *
 *													    *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 21/01/08 | C&G			| Cration et code des    | *
 * |          |			   | fonctions: 			  | *
 * |          |            | -Moteur_Init			  | *
 * |          |            | -Moteur_CommandeG		  | *
 * |          |            | -Moteur_CommandeD	      | *
 * |--------------------------------------------------| *
 * |  7/02/08 |    C&G     | Mise  jour du code      | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques : commande moteur en [-1023;+1023]        *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include <timers.h>			// Librairie de configuration des timers
#include <pwm.h>			// Librairie de gestion de config pwm

#include "Moteur.h"

/*-------------------------------------------------------------------
 *Fonction Moteur_Init()
 * Cette fonction initialise les moteurs
	- configurations des ports
	- configuration des timers
	- configuration des PWM G&D
*/
void Moteur_Init(void)
{
	// Configurations des ports en sorties
	TRIS_PWMG =0;				// PWM du moteur gauche en sortie
	TRIS_PWMD =0;				// PWM du moteur droit en sortie
	TRIS_DIRG =0;				// Sens du moteur gauche en sortie
	TRIS_DIRD =0;				// Sens du moteur droit en sortie

	OpenTimer2 (TIMER_INT_OFF &	T2_PS_1_1 &	T2_POST_1_1); 	// Timer 2 utilis pour le le PWM

	OpenPWM1(0xFF);				// Frquence du PWM gauche  : 19531 Hz 
	OpenPWM2(0xFF);				// Frquence du PWM droit : 19531 Hz 
	SetDCPWM1(0x00);			// Arrete le moteur gauche
	SetDCPWM2(0x00);			// Arrete le moteur droit
}
//-------------------------------------------------------------------------------------------



/*-------------------------------------------------------------------
 * Fonction Moteur_CommandeG() qui fait rouler le robot
 * Envoi d'une valeur comprisse entre -1023 et +1024
	* -1023 ->  fond en arriere 
	* 0 	-> arret du moteur
	* +1024 ->  fond en avant
*/
void Moteur_CommandeG(int vit)
{	
	if(vit>1023) vit=1023;
	if(vit<-1023) vit=-1023;
	
	if (vit==0)	SetDCPWM1(0);	// Trait  part pour viter les changements de sens bruteaux
	
	if (vit>0) 
	{
		SetDCPWM1(vit);			// Rgle le rapport cyclique du PWM -> Vitesse du moteur
		DIRG=0;					// sens positif	
	}
	
	if (vit<0)
	{
		SetDCPWM1(-vit);		// Rgle le rapport cyclique du PWM -> Vitesse du moteur
		DIRG=1;					// sens ngatif
	}

}

//-------------------------------------------------------------------------------------------

/*-------------------------------------------------------------------
 * Fonction Moteur_CommandeD() qui fait rouler le robot
 * Envoi d'une valeur comprisse entre -1023 et +1024
	* -1023 ->  fond en arriere 
	* 0 	-> arret du moteur
	* +1024 ->  fond en avant
*/
void Moteur_CommandeD(int vit)
{
	if(vit>1023) vit=1023;
	if(vit<-1023) vit=-1023;
	if (vit==0) SetDCPWM2(0);	// Trait  part pour viter les changements de sens bruteaux
	if(vit>0)
	{
		SetDCPWM2(vit);			// Rgle le rapport cyclique du PWM -> Vitesse du moteur
		DIRD=1;					// sens positif
	}
	if (vit<0)
	{
		SetDCPWM2(-vit);		// Rgle le rapport cyclique du PWM -> Vitesse du moteur
		DIRD=0;					// sens ngatif
	}	
}