// Fichier Moteur.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   		de la commande Moteur de propulsion			*
 *                                                      *
 * .Date de cration : 21/01/08                         *
 * .Auteurs : Came Mikael Et Guibert David              *       
 *                                                      *
 * .Descriptions :                                      *
 *													    *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 21/01/08 | C&G			| Cration et code des    | *
 * |          |			   | fonctions: 			  | *
 * |          |            | -Moteur_Init			  | *
 * |          |            | -Moteur_CommandeG		  | *
 * |          |            | -Moteur_CommandeD	      | *
 * |--------------------------------------------------| *
 * |  7/02/08 |    C&G     | Mise  jour du code      | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques : commande moteur en [-1023;+1023]        *
 *                                                      *
 ********************************************************/

#ifndef __MOTEUR_H__
#define __MOTEUR_H__

//-----------------------------

// Dfinition des broches utile aux moteurs
#define PWMG 		PORTCbits.RC2
#define PWMD 		PORTCbits.RC1
#define DIRG 		PORTBbits.RB4
#define DIRD 		PORTBbits.RB5

#define TRIS_PWMG	TRISCbits.TRISC2
#define TRIS_PWMD	TRISCbits.TRISC1
#define TRIS_DIRG	TRISBbits.TRISB4
#define TRIS_DIRD	TRISBbits.TRISB5

//------------------------------

// Prototype des fonctions


/*-------------------------------------------------------------------
 *Fonction Moteur_Init()
 * Cette fonction initialise les moteurs
	- configurations des ports
	- configuration des timers
	- configuration des PWM G&D
*/
void Moteur_Init(void);


/*-------------------------------------------------------------------
 * Foncttion Moteur_CommandeG()
 * Commande le moteur de la roue gauche
 * L'argument est une valeur entre -1023 et +1023
 *  -1023 : le moteur tourne a fond en arire
 *  0     : Le moteur est  l'arrt
 *  +1023 : le moteur tourne a fond en avant
*/
void Moteur_CommandeG(int vit);

/*-------------------------------------------------------------------
 * Foncttion Moteur_CommandeD()
 * Commande le moteur de la roue droite
 * L'argument est une valeur entre -1023 et +1023
 *  -1023 : le moteur tourne a fond en arire
 *  0     : Le moteur est  l'arrt
 *  +1023 : le moteur tourne a fond en avant
*/
void Moteur_CommandeD(int vit);





#endif