// Fichier PI.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   			d'asservissement du robot				*
 *                                                      *
 * .Date de cration : 23/01/08                         *
 * .Auteurs : P. Lucidarme					            *       
 *                                                      *
 * .Remarques : 										*
 *                                                      *
 *                                                      *
 ********************************************************/

#include <timers.h>
#include <p18f452.h>
#include <math.h>

#include "PI.h"
#include "codeur.h"
#include "moteur.h"
#include "led.h"


unsigned int 		PI_Codeur_val_G_vieux;					// Ancienne valeur du codeur gauche
unsigned int 		PI_Codeur_val_D_vieux;					// Ancienne valeur du codeur droit

static int	 	ErrorLineaireOld=0;							// Erreur sur la vitesse linaire de l'chantillon prcdent
static int 		ErrorAngulaireOld=0;						// Erreur sur la vitesse angulaire de l'chantillon prcdent
static float 	CommandeLineaireOld=0;						// Erreur sur la vitesse linaire de l'chantillon prcdent
static float 	CommandeAngulaireOld=0;						// Erreur sur la vitesse angulaire de l'chantillon prcdent

static int	 	ErrorGaucheOld=0;							// Erreur sur la vitesse linaire de l'chantillon prcdent
static int 		ErrorDroiteOld=0;							// Erreur sur la vitesse angulaire de l'chantillon prcdent
static float 	CommandeGaucheOld=0;						// Erreur sur la vitesse linaire de l'chantillon prcdent
static float 	CommandeDroiteOld=0;						// Erreur sur la vitesse angulaire de l'chantillon prcdent

	
static float 	VitesseLineaireOld=0;
static int		NError=0;




/*-------------------------------------------------------------------
 * Initialise l'asservissement du robot
 */
void PI_Init(void)
{
	Codeur_Lire_val( &PI_Codeur_val_G_vieux , &PI_Codeur_val_D_vieux);	// Lit les codeurs pour initaliser les valeurs  n-1
	OpenTimer0 (	TIMER_INT_OFF	&									// Pas d'interruption
					T0_16BIT 		&									// Compteur 16 bits
					T0_SOURCE_INT 	&									// Timer sur quartz
					T0_PS_1_1	); 										// Pas de prscaler
					
	WriteTimer0 (T_TIMER_0);											// Charge le timer avec la valeur initiale
 	ErrorLineaireOld=0;													// Erreur sur la vitesse linaire de l'chantillon prcdent
	ErrorAngulaireOld=0;												// Erreur sur la vitesse angulaire de l'chantillon prcdent
 	CommandeLineaireOld=0.0;											// Erreur sur la vitesse linaire de l'chantillon prcdent
 	CommandeAngulaireOld=0.0;											// Erreur sur la vitesse angulaire de l'chantillon prcdent
	
 	ErrorGaucheOld=0;													// Erreur sur la vitesse linaire de l'chantillon prcdent
	ErrorDroiteOld=0;													// Erreur sur la vitesse angulaire de l'chantillon prcdent
 	CommandeGaucheOld=0.0;												// Erreur sur la vitesse linaire de l'chantillon prcdent
 	CommandeDroiteOld=0.0;												// Erreur sur la vitesse angulaire de l'chantillon prcdent	
	
 	VitesseLineaireOld=0;												// Variable pour la dtection d'erreur
	NError=0;															// Nombre d'erreurs conscutives  zro
}	



/*-------------------------------------------------------------------
 * Rinitialise l'asservissement du robot
 */
void PI_Reset(void)
{
	Codeur_Lire_val( &PI_Codeur_val_G_vieux , &PI_Codeur_val_D_vieux);	// Lit les codeurs pour initaliser les valeurs  n-1					
	WriteTimer0 (T_TIMER_0);											// Charge le timer avec la valeur initiale
 	ErrorLineaireOld=0;													// Erreur sur la vitesse linaire de l'chantillon prcdent
	ErrorAngulaireOld=0;												// Erreur sur la vitesse angulaire de l'chantillon prcdent
 	CommandeLineaireOld=0.0;											// Erreur sur la vitesse linaire de l'chantillon prcdent
 	CommandeAngulaireOld=0.0;											// Erreur sur la vitesse angulaire de l'chantillon prcdent	
 	VitesseLineaireOld=0;												// Variable pour la dtection d'erreur
	NError=0;															// Nombre d'erreurs conscutives  zro
}	


/*-------------------------------------------------------------------
 * Rinitialise l'asservissement du robot en commande directe asservie
 */
void PI_Direct_Reset(void)
{
	Codeur_Lire_val( &PI_Codeur_val_G_vieux , &PI_Codeur_val_D_vieux);	// Lit les codeurs pour initaliser les valeurs  n-1					
	WriteTimer0 (T_TIMER_0);											// Charge le timer avec la valeur initiale
 	ErrorGaucheOld=0;													// Erreur sur la vitesse linaire de l'chantillon prcdent
	ErrorDroiteOld=0;													// Erreur sur la vitesse angulaire de l'chantillon prcdent
 	CommandeGaucheOld=0.0;												// Erreur sur la vitesse linaire de l'chantillon prcdent
 	CommandeDroiteOld=0.0;												// Erreur sur la vitesse angulaire de l'chantillon prcdent	
}	


/*-------------------------------------------------------------------
 * Actualise les donnes de l'asservissement pendant les phase d'arrt
 * ou de commande directe
 */
void PI_Stand_By(void)
{
	Codeur_Lire_val( &PI_Codeur_val_G_vieux , &PI_Codeur_val_D_vieux);	// Lit les codeurs pour initaliser les valeurs  n-1	
			
	VitesseLineaireOld=0;										// Pour le calcul du modle

	ErrorLineaireOld=0;											// Reset l'erreur linaire
	CommandeLineaireOld=0;										// Reset la commande linaire
	
	ErrorAngulaireOld=0;										// Reset l'erreur angulaire
 	CommandeAngulaireOld=0;										// Reset la commande angulaire
	
}	


/*-------------------------------------------------------------------
 * Lit les codeurs et calcul les vitesse linaire et angulaire 
 * Pointeur vers les vitesses linaires et angulaires
 */
void Mesure_Vitesses (int* Vitesse_Lineaire, int* Vitesse_Angulaire )
{
	unsigned int 		PI_Codeur_val_G;						// Valeur courante du codeur gauche
	unsigned int 		PI_Codeur_val_D;						// Valeur courante du codeur droit
	int 				VitG,VitD;								// Vitesse des roues droites et gauche
	
	Codeur_Lire_val( &PI_Codeur_val_G , &PI_Codeur_val_D);		// Lit les codeurs et sauvegarde dans les variables	
	
	VitG= PI_Codeur_val_G - PI_Codeur_val_G_vieux;				// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Gauche
	VitD= PI_Codeur_val_D_vieux - PI_Codeur_val_D; 				// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Droit

	*Vitesse_Lineaire =VitG+VitD;								// Calcule la vitesse linaire
	*Vitesse_Angulaire=VitD-VitG;								// Calcule la vitesse angulaire

	PI_Codeur_val_G_vieux=PI_Codeur_val_G;						// Sauvegarde de la valeur du codeur gauche actuel pour le prochain appel de la fonction
	PI_Codeur_val_D_vieux=PI_Codeur_val_D;						// Sauvegarde de la valeur du codeur droit actuel pour le prochain appel de la fonction
}


/*-------------------------------------------------------------------
 * Asservissement en vitesse linaire et angulaire
 * Attends le dbordement du timer et asservie en vitesse le robot
 * Les paramtres reus sont les consignes linaire et angulaire
 * La fonction retourne le nombre d'erreur conscutives
 */

unsigned int PI_Vitesse(int ConsigneVitesseLineaire, int ConsigneVitesseAngulaire)
{

	int				VitesseLineaire,VitesseAngulaire;			// Vitesses mesures
	int 			ErrorLineaire,ErrorAngulaire;				// Erreurs linaire et angulaire
	float 			CommandeLineaire,CommandeAngulaire;			// Commandes linaire et angulaire courantes
	float 			VitesseLineaireEstime;						// Vitesse linaire estime grace au model
	
	while (!INTCONbits.TMR0IF);									// Attends le dbordement du timer (10ms)
	WriteTimer0 (T_TIMER_0);									// Recharge le timer
	INTCONbits.TMR0IF=0;										// Remet  zro le bit de dbordement
	LED_VERTE=!LED_VERTE;										// Change l'tat de la led verte 
		
	Mesure_Vitesses (&VitesseLineaire,&VitesseAngulaire);		// Mesure les vitesses
			
	
	ErrorLineaire =ConsigneVitesseLineaire-VitesseLineaire;		// Calcul l'erreur en vitesse linaire
	ErrorAngulaire=ConsigneVitesseAngulaire-VitesseAngulaire;	// Calcule l'erreur en vitesse angulaire
	
	VitesseLineaireEstime=	0.00750634172*CommandeLineaireOld	// Simule le modle du robot
							+0.95069600031*VitesseLineaireOld;	
														
	VitesseLineaireOld=VitesseLineaire;							// Mmorise l'ancienne valeur
	if (fabs(VitesseLineaire-VitesseLineaireEstime)>SensibError)// Si l'erreur dpasse le seuil de sensibilit ...
		NError++;												// On incrmente le nombre d'erreur
	else 
		NError=0;												// On remet le compteur  zro

	CommandeLineaire=	CommandeLineaireOld;					// Calcul le PI pour la vitesse linaire
	CommandeLineaire+=	(KiLin-KpLin)*(float)ErrorLineaireOld;
	CommandeLineaire+=	KpLin*(float)ErrorLineaire;	
							
	CommandeAngulaire=	CommandeAngulaireOld;					// Calcul le PI pour la vitesse angulaire
	CommandeAngulaire+=(KiAng-KpAng)*(float)ErrorAngulaireOld;
	CommandeAngulaire+=	KpAng*(float)ErrorAngulaire;	

	if (CommandeLineaire> 2046.0) CommandeLineaire= 2046.0;			// Bride la commande linaire
	if (CommandeLineaire<-2046.0) CommandeLineaire=-2046.0;			// Bride la commande linaire
	if (CommandeAngulaire> 2046.0) CommandeAngulaire= 2046.0;		// Bride la commande angulaire
	if (CommandeAngulaire<-2046.0) CommandeAngulaire=-2046.0;		// Bride la commande angulaire
		
	
	ErrorLineaireOld=ErrorLineaire;								// Mmorise l'erreur linaire
	CommandeLineaireOld=CommandeLineaire;						// Mmorise la commande linaire
	
	ErrorAngulaireOld=ErrorAngulaire;							// Mmorise l'erreur angulaire
 	CommandeAngulaireOld=CommandeAngulaire;						// Mmorise la commande angulaire
	
	Motor_Commande_Vitesse ((int)CommandeLineaire,				// Balance la commande sur les moteurs
							(int)CommandeAngulaire);
	
	return (NError);
}	



/*-------------------------------------------------------------------
 * Asservissement en vitesse direct des roues gauches et droite
 * Attends le dbordement du timer et asservie en vitesse le robot
 * Les paramtres reus sont les consignes linaire et angulaire
 * La fonction retourne le nombre d'erreur conscutives
 */

void PI_Direct_Vitesse(int ConsigneGauche, int ConsigneDroite)
{

	int				VitesseGauche,VitesseDroite;			// Vitesses mesures
	int 			ErrorGauche,ErrorDroite;				// Erreurs linaire et angulaire
	float 			CommandeGauche,CommandeDroite;			// Commandes linaire et angulaire courantes

	unsigned int 	PI_Codeur_val_G;						// Valeur courante du codeur gauche
	unsigned int 	PI_Codeur_val_D;						// Valeur courante du codeur droit

	
	while (!INTCONbits.TMR0IF);									// Attends le dbordement du timer (10ms)
	WriteTimer0 (T_TIMER_0);									// Recharge le timer
	INTCONbits.TMR0IF=0;										// Remet  zro le bit de dbordement
	LED_VERTE=!LED_VERTE;										// Change l'tat de la led verte 
	
		
	Codeur_Lire_val( &PI_Codeur_val_G , &PI_Codeur_val_D);		// Lit les codeurs et sauvegarde dans les variables	
	
	VitesseGauche= PI_Codeur_val_G - PI_Codeur_val_G_vieux;		// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Gauche
	VitesseDroite= PI_Codeur_val_D_vieux - PI_Codeur_val_D; 	// On rcupere le nombre d'impulsions depuis la derniereIT du moteur Droit

	PI_Codeur_val_G_vieux=PI_Codeur_val_G;						// Sauvegarde de la valeur du codeur gauche actuel pour le prochain appel de la fonction
	PI_Codeur_val_D_vieux=PI_Codeur_val_D;						// Sauvegarde de la valeur du codeur droit actuel pour le prochain appel de la fonction

	
	ErrorGauche =ConsigneGauche-VitesseGauche;					// Calcul l'erreur en vitesse linaire
	ErrorDroite =ConsigneDroite-VitesseDroite;					// Calcule l'erreur en vitesse angulaire
	
	CommandeGauche =	CommandeGaucheOld;						// Calcul le PI pour la vitesse gauche
	CommandeGauche+=	(KiLin-KpLin)*(float)ErrorGaucheOld;
	CommandeGauche+=	KpLin*(float)ErrorGauche;	
							
	CommandeDroite =	CommandeDroiteOld;						// Calcul le PI pour la vitesse droite
	CommandeDroite+=	(KiLin-KpLin)*(float)ErrorDroiteOld;
	CommandeDroite+=	KpLin*(float)ErrorDroite;	

	if (CommandeGauche> 1022.0) CommandeGauche= 1022.0;				// Bride la commande linaire
	if (CommandeGauche<-1022.0) CommandeGauche=-1022.0;				// Bride la commande linaire
	if (CommandeDroite> 1022.0) CommandeDroite= 1022.0;				// Bride la commande angulaire
	if (CommandeDroite<-1022.0) CommandeDroite=-1022.0;				// Bride la commande angulaire
		
	
	ErrorGaucheOld=ErrorGauche;									// Mmorise l'erreur linaire
	CommandeGaucheOld=CommandeGauche;							// Mmorise la commande linaire
	
	ErrorDroiteOld=ErrorDroite;									// Mmorise l'erreur angulaire
 	CommandeDroiteOld=CommandeDroite;							// Mmorise la commande angulaire
	

	Moteur_CommandeG((int)CommandeGauche);			// Balance la commande sur les moteurs
	Moteur_CommandeD((int)CommandeDroite);

}	






/*-------------------------------------------------------------------
 * Commande les moteurs du robot en vitesse
 * Le premier paramtre est la vitesse d'avancement linaire du robto
 * Le seconde la vitesse angulaire
 * Mg=VLin+Vang
 * Md=VLin-Vang
*/
void Motor_Commande_Vitesse (int VitesseLineaire, int VitesseAngulaire)
{
	Moteur_CommandeG(VitesseLineaire-VitesseAngulaire);
	Moteur_CommandeD(VitesseLineaire+VitesseAngulaire);
}



