// Fichier PI.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   			d'asservissement du robot				*
 *                                                      *
 * .Date de cration : 23/01/08                         *
 * .Auteurs : P. Lucidarme					            *       
 *                                                      *
 * .Remarques : 										*
 *                                                      *
 *                                                      *
 ********************************************************/



#ifndef __PI__H__
#define __PI__H__

#define				T_TIMER_0			15535					// Chargement du timer 0 pour arriver  une priode de 10ms
#define				KpLin				6.56831275				// Coefficient proportionnel de l'asservissement en vitesse linaire
#define				KiLin				1.0						// Coefficient Integral de l'asservissement en vitesse linaire (Valeur exacte : 0.32384409)
#define				KpAng				8.58343803				// Coefficient proportionnel de l'asservissement en vitesse angulaire
#define				KiAng				0.8						// Coefficient Integral de l'asservissement en vitesse linaire (Valeur exacte : 0.44853104)

#define				SensibError			8						// Sensibilit pour le dtection d'erreur (plus c'est faible, plus c'est sensible)
#define				NSampleError		5						// Nombre d'chantillons utiliss pour la ddection d'erreurs (Plus c'est grand plus c'est long)




/*-------------------------------------------------------------------
 * Initialise l'asservissement du robot
 */
void PI_Init(void);
	



/*-------------------------------------------------------------------
 * Rinitialise l'asservissement du robot
 */
void PI_Reset(void);



/*-------------------------------------------------------------------
 * Rinitialise l'asservissement du robot en commande directe asservie
 */
void PI_Direct_Reset(void);


/*-------------------------------------------------------------------
 * Actualise les donnes de l'asservissement pendant les phase d'arrt
 * ou de commande directe
 */
void PI_Stand_By(void);



/*-------------------------------------------------------------------
 * Lit les codeurs et calcul les vitesse linaire et angulaire 
 * Pointeur vers les vitesses linaires et angulaires
 */
void Mesure_Vitesses (int* Vitesse_Lineaire, int* Vitesse_Angulaire);





/*-------------------------------------------------------------------
 * Asservissement en vitesse linaire et angulaire
 * Attends le dbordement du timer et asservie en vitesse le robot
 * Les paramtres reus sont les consignes linaire et angulaire
 * La fonction retourne le nombre d'erreur conscutives
 */
unsigned int PI_Vitesse(int ConsigneVitesseLineaire, int ConsigneVitesseAngulaire);



/*-------------------------------------------------------------------
 * Asservissement en vitesse direct des roues gauches et droite
 * Attends le dbordement du timer et asservie en vitesse le robot
 * Les paramtres reus sont les consignes linaire et angulaire
 * La fonction retourne le nombre d'erreur conscutives
 */

void PI_Direct_Vitesse(int ConsigneGauche, int ConsigneDroite);



/*-------------------------------------------------------------------
 * Commande les moteurs du robot en vitesse
 * Le premier paramtre est la vitesse d'avancement linaire du robto
 * Le seconde la vitesse angulaire
 * Mg=VLin+Vang
 * Md=VLin-Vang
*/
void Motor_Commande_Vitesse (int VitesseLineaire, int VitesseAngulaire);



#endif