// Fichier codeur.C
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   			de la gestion des codeurs				*
 *                                                      *
 * .Date de cration : 23/01/08                         *
 * .Auteurs : Came Mikael Et Guibert David              *       
 *                                                      *
 * .Descriptions :                                      *
 *													    *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 23/01/08 | C&G		   | Cration et code des     | *
 * |          |			   | fonctions: 			  | *
 * |          |            | Codeur_Init(void) 		  | *
 * |          |            | Codeur_Gauche(void) 	  | *
 * |          |            | Codeur_Droite(void) 
 *|			  |			   |Codeur_Lire_val(unsigned int *pValG,unsigned int* pValD)					 	  | *
 * |--------------------------------------------------| *
 * |  7/02/08 |    C&G     | Mise  jour du code      | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques : 										*
 *                                                      *
 *		RD : Gle la valeur des codeurs pour une*		*
 *  lecture plus facile									*
 *														*
 *		D/G : Permet la selection du codeur a lire		*	
 *					1 : Codeur Droite					*	
 *					0 : Codeur Gauche					*	
 *		H/L	: Permet la selection de la partie de 		*
 *	l'octet a lire										*
 *	car la valeur des codeurs est sur 16 bits			*
 *					1 : Parties hautes					*
 *					0 : Parties basses					*
 *		DATA : Port  ou circule la valeur des codeurs	* 
 *                                                      *
 ********************************************************/


//-------------------------------------------------------------------
#include <p18f452.h>			// Librairie du PIC utilis

#include "codeur.h"
//-------------------------------------------------------------------



/*-------------------------------------------------------------------
 * Fonction Codeur_Init()
 * Initialise les broches des codeurs en entrees/sorties
 *  0: en sorties
 *  1: en entrees 
*/
void Codeur_Init(void)
{
	// Configurations des ports en sorties/entrees
	TRIS_CODEUR_RD=0;			// RD en sortie
	TRIS_CODEUR_DG =0;			// DG en sortie	
	TRIS_CODEUR_HL =0;			// HL en sortie
	TRIS_CODEUR_DATA =0xFF;		// DATA en entree

}


/*-------------------------------------------------------------------
 * Fonction Codeur_Gauche()
 * Retourne la valeur du codeur gauche
*/
unsigned int Codeur_Gauche(void)
{
	unsigned int CodeurVal=0;

	CODEUR_DG=0;							// Slectionne le codeur gauche
	CODEUR_HL=1;							// Slectionne l'octet de poids fort
	CodeurVal=CODEUR_DATA; 					// Enregistre le poids fort
	CODEUR_HL=0;							// Slectionne l'octet de poids faible
	CodeurVal=(CodeurVal<<8)+CODEUR_DATA; 	// Enregistre la valeur du codeur

	return	CodeurVal;						// Retourne la valeur du codeur gauche
}


/*-------------------------------------------------------------------
 * Fonction Codeur_Droit()
 * Retourne la valeur du codeur droit
*/
unsigned int Codeur_Droit(void)
{
	unsigned int CodeurVal=0;

	CODEUR_DG=1;							// Slectionne le codeur droit
	CODEUR_HL=1;							// Slectionne l'octet de poids fort
	CodeurVal=CODEUR_DATA; 					// Enregistre le poids fort
	CODEUR_DG=1;
	CODEUR_HL=0;							// Slectionne l'octet de poids faible
	CodeurVal=(CodeurVal<<8)+CODEUR_DATA; 	// Enregistre la valeur du codeur

	return	CodeurVal;						// Retourne la valeur du codeur droit

}

/*-------------------------------------------------------------------
 * Fonction Codeur_Lire_val(unsigned int *pValG,unsigned int* pValD)
 * Lit les codeurs gauche et droit et retourne les valeurs dans les variables passs en arguments
	- PvalG pointe sur la variable ou va etre enregistr la valeur du codeur GAUCHE
	- PvalD pointe sur la variable ou va etre enregistr la valeur du codeur DROIT
*/

void Codeur_Lire_val(unsigned int *pValG,unsigned int* pValD)
{
	CODEUR_RD=1;							// Gle le codeur
		*pValG=Codeur_Gauche();				// Lecture de la valeur du codeur droit
		*pValD=Codeur_Droit();				// Lecture de la valeur du codeur gauche
	CODEUR_RD=0;							// Dgle le codeur

}