// Fichier codeur.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   			de la gestion des codeurs				*
 *                                                      *
 * .Date de cration : 23/01/08                         *
 * .Auteurs : Came Mikael Et Guibert David              *       
 *                                                      *
 * .Descriptions :                                      *
 *													    *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 23/01/08 | C&G		   | Cration et code des     | *
 * |          |			   | fonctions: 			  | *
 * |          |            | Codeur_Init(void) 		  | *
 * |          |            | Codeur_Gauche(void) 	  | *
 * |          |            | Codeur_Droite(void)	  | *
 * |--------------------------------------------------| *
 * |  7/02/08 |    C&G     | Mise  jour du code      | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques : 										*
 *                                                      *
 *		RD : Gle la valeur des codeurs pour une		*
 *  lecture plus facile									*
 *														*
 *		D/G : Permet la selection du codeur a lire		*	
 *					1 : Codeur Droite					*	
 *					0 : Codeur Gauche					*	
 *		H/L	: Permet la selection de la partie de 		*
 *	l'octet a lire										*
 *	car la valeur des codeurs est sur 16 bits			*
 *					1 : Parties hautes					*
 *					0 : Parties basses					*
 *		DATA : Port  ou circule la valeur des codeurs	* 
 *                                                      *
 ********************************************************/

#ifndef __CODEUR_H__
#define __CODEUR_H__


// Dfinition des broches utile  la gestion des codeurs
#define CODEUR_RD   		PORTBbits.RB3
#define CODEUR_HL   		PORTAbits.RA0
#define CODEUR_DG   		PORTBbits.RB2
#define CODEUR_DATA   		PORTD


#define TRIS_CODEUR_RD 		TRISBbits.TRISB3
#define TRIS_CODEUR_HL 		TRISAbits.TRISA0
#define TRIS_CODEUR_DG 		TRISBbits.TRISB2
#define TRIS_CODEUR_DATA 	TRISD



/*-------------------------------------------------------------------
 * Fonction Codeur_Init()
 * Initialise les broches des codeurs en entres/sorties
 *  0: en sorties
 *  1: en entres 
*/
void Codeur_Init(void);

/*-------------------------------------------------------------------
 * Fonction Codeur_Gauche()
 * Retourne la valeur du codeur gauche
*/
unsigned int Codeur_Gauche(void);



/*-------------------------------------------------------------------
 * Fonction Codeur_Droit()
 * Retourne la valeur du codeur droit
*/
unsigned int Codeur_Droit(void);

/*-------------------------------------------------------------------
 * Fonction Codeur_Lire_val(unsigned int *pValG,unsigned int* pValD)
 * Lit les codeurs gauche et droit et retourne les valeurs dans les 
	variables passs en arguments
*/
void Codeur_Lire_val(unsigned int *pValG,unsigned int* pValD);

#endif