// Fichier model.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *	   			de calcul de la position du robot		*
 *                                                      *
 * .Date de cration : 23/01/08                         *
 * .Auteurs : P. Lucidarme					            *       
 *                                                      *
 * .Remarques : 										*
 *                                                      *
 *                                                      *
 ********************************************************/

#ifndef __MODEL_H__
#define __MODEL_H__


// Mensuration du robot
#define 	r 				28.45					// Rayon de la roue codeuse en mm
#define 	l 				133.25					// Demi entraxe entre les deux roues codeuse en mm
#define 	K1				((PI*r)/(2.0*400.0))	// Coef pour les coordonnes X, Y 
#define 	K2				((PI*r)/(2.0*400.0*l))	// Coef pour le differentiel de l'angle du robot
#define		PI				3.141592653589793		// Constante Pi

/* Structure d'un point sur l'aire de jeu
 * Coordonne en X et Y
	- Pour les coordonnes du robot
	- Pour les coordonnes de la cible traqu 
*/
struct position
{
	float X;					// Contient la coordonne en X 
	float Y;					// Contient la coordonne en X 
	float Psi;					// Contient l'orientation du robot
};




/*-------------------------------------------------------------------
 * Initialise le model gomtrique du robot
 * Reoit en paramtre la position/orientation de dpart du robot
*/
void Model_Init(float X, float Y, float Psi);




/*-------------------------------------------------------------------
 * Lit les codeurs et met  jour le modle gomtrique
 * Aucun paramtre
 * Dure d'execution minimum : 500s
 * Dure d'xecution maximum : 2.5ms
*/
void Model_Cinematique (void);




/*-------------------------------------------------------------------
 * Calcul l'angle entre le robot et la cible
 * Les paramtres passs sont XCible et YCible
 * Si le robot est confondu avec la cible, la fonction retourne 0
*/
float Angle_Robot_Cible(float XCible, float YCible);




/*-------------------------------------------------------------------
 * Retourne le carr de la distance entre le robot et la cible
 * Les paramtres passs sont XCible et YCible
 */
float Distance_Cible(float XCible, float YCible);

#endif