// Fichier Serial.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>			// Librairie du PIC utilis
#include <usart.h>				// Librairie pour la liaison srie
#include <stdio.h>				// Pour utiliser le printf 
#include <delays.h>
#include <string.h>

#include "serial.h"	
#include "led.h"




//----------------------------------------------------------------------------------------
volatile char	Serial_Flag=0;				// Variable partag avec le main qui signal une trame complete 	
volatile char 	CopieTrame[MAX_TRAME];		// Copie de la trame, une fois qu'elle est complte
//----------------------------------------------------------------------------------------







// --------------------------------------------------------------------------
/* Initialise le port de la led rouge en sortie
 *  Pas de paramtre
*/
void Serial_Init()
{
	TRIS_RX=1;								// RX en entre (rception)
	TRIS_TX=0;								// TX en sortie (transmission)

	IPR1bits.RCIP=1;						// Dfini l'IT de la liasion srie comme priorit haute 

	// Configure la liaison srie  57600 Bauds:
	OpenUSART	(	USART_TX_INT_OFF	&	// Pas d'interruption sur l'mission
					USART_RX_INT_ON		&	// Interruption sur la rception
					USART_ASYNCH_MODE	&	// Mode asynchrone
					USART_EIGHT_BIT		& 	// Transmission sur 8 bits (1 octet)
					USART_CONT_RX		&	// Reception continue
					USART_BRGH_HIGH		,	// Haute vitesse
					21);					// 21 = (20e6 / (16*57600) -1
 
	while (DataRdyUSART())					// Tant qu'il y a des donnes sur le buffer
		ReadUSART();						// on lit les donnes
}	




// --------------------------------------------------------------------------
/* Calcul le CRC de la trame passe en paramtre
 *  Le premier paramtre est un pointeur vers le premier caractre de la trame
 *  Le second est le nombre de caractres
*/
unsigned char CRC (unsigned char* Trame,int NbCar)
{
	int i;
	unsigned char CRC=0;
	for (i=0;i<NbCar;i++)
		CRC^=Trame[i];
	return (CRC);		
}	



// --------------------------------------------------------------------------
/* Envoyer la trame sur la liaison srie
 *  Le premier est un pointeur vers le premier caractre de la trame
 *  Le second paramtre est le nombre de caractre  envoyer 
*/
void Envoyer_Trame(unsigned char* Trame, char NbCar)
{
	int i=0;
	while (BusyUSART());			// Tant que le module est occup, on attends
	for (i=0;i<NbCar;i++)			// Parcours tous les caractres
	{
		WriteUSART (Trame[i]);		// On envoie le caractre courant
		while (BusyUSART());		// Tant que le module est occup, on attends
		DELAY_USART					// Petit dlai pour viter les problmes
	}	
}	



/* Reception d'une valeur envoyer par le PC grace  la liaison srie
 * A chaque rception l'IT et appeler 
 * Changement de l'tat de la led
*/
#pragma interrupt InterruptSerial	
void InterruptSerial(void)
{
	static unsigned char Trame[MAX_TRAME];		// Tableau dans lequel on memorise la trame
	static unsigned char Nb=0;					// Indice du caractre courant dans la trame
	int i;										// Variable utilise pour la copie de la trame
	

	Trame[Nb]=ReadUSART();						// On lit le caractre 
												// (quoi qu'il en soit au moins pour ractiver l'interruption)
	
	if (RCSTAbits.OERR) 						// Si la liaison srie est en OVERRUN
	{
		RCSTAbits.CREN=0;						// Eteind le rcepteur ...
		RCSTAbits.CREN=1;						// ... pour le redmarrer				
		Nb = 0;									// Vider la trame
		return;
	}	


	if (Serial_Flag==1)							// On a dj une trame non traite
	{
		Nb=0;									// La nouvelle est perdue
		return;									// On quitte
	}
	
	
	if (Nb==0)									// Si c'est le premier caractre
	{
		switch (Trame[Nb])						// On vrifie que c'est un caractre identifi
		{
			case 'D': Nb++; return;				// C'est la commande Direct   		, on incrmente Nb
			case 'P': Nb++; return;				// C'est la commande direct polaire , on incrmente Nb
			case 'A': Nb++; return;				// C'est la commande Arret   		, on incrmente Nb															
			case 'S': Nb++; return;				// C'est la commande State	 		, on incrmente Nb		
			case 'F': Nb++; return;				// C'est la commande Force	 		, on incrmente Nb		
			case 'G': Nb++; return;				// C'est la commande Goto			, on incrmente Nb		
			case 'O': Nb++; return;				// C'est la commande Oriente	  	, on incrmente Nb		
			case 'T': Nb++; return;				// C'est la commande ToutDroit		, on incrmente Nb	
			case 'B': Nb++; return;				// C'est la commande directe asservie		, on incrmente Nb	
			default : return;					// Si ce n'est pas une commande connue, on ne fait rien
		}		
	}
	

	
	Nb++;										// On est en cours de rception de trame, on passe au suivant
	
	
	if (Nb==MAX_TRAME)							// Si on a reu une trame complte
	{
		for (i=0;i<MAX_TRAME;i++)				// Copie la trame
			CopieTrame[i]=Trame[i];				// Pour la partager avec le programme principal
		Serial_Flag=1;							// On passe le flag  1
		Nb=0;		
	}
	
}
