#include "trigo.h"


// Table des coefficients relatifs aux polynomes interpolateurs 
//  des fonctions considres:
float Trigo_Mat_sin[rec_sin]={
-1.92370246639E-7	, 3.42962638388E-6	,-1.36689877454E-6  ,
-0.00019666870938	,-1.44048404803E-6	, 0.00833410130649	,
-2.56156670335E-7 	,-0.16666661669		,-5.01143002096E-9  , 
 1.0000000002		,-1.27398092076E-12	};

float Trigo_Mat_exp[rec_exp]={
 2.78597386348E-6  	, 0.0000251345519324, 0.000198399964585	, 
 0.00138874864811	, 0.00833333551987	, 0.0416666907509	, 
 0.166666666535		, 0.499999998553	, 1.				,
 1.00000000001		};

float Trigo_Mat_log[rec_log]={
 1.8750679912		,-14.2305757565		, 48.4608383358		,
-97.6533222865		, 129.28327909		,-118.087752984		, 
 76.1972936393		,-35.3221671175		, 12.650130592		,
-3.1727915005		};

float Trigo_Mat_sqr[rec_sqr]={
 0.155566847059		,-1.18905601862		, 4.08694713883		,
-8.34005874073		, 11.2424647178		,-10.5590526927		,
 7.1517777844		,-3.68250447355		, 1.97763187612		, 
 0.156283561672		};

float Trigo_Mat_atn[rec_atn]={
-0.0235918799262	, 0.0953409945536	,-0.103609345687	,
-0.0904724723927	, 0.255917957961	,-0.0175631491321	,
-0.330390415431		,-0.000241002322412	, 1.00000747423		,
-3.75392539276E-8 	};





// retourne les valeur +INF / -INF et NAN
float Trigo_returnINF_LO(void) 
{
	char* c;
	float f;
	c = (char*)&f;
	c[0] = 0;		c[1] = 0;
	c[2] = 0x80;	c[3] = 0xFF;
	return f;
}



float Trigo_returnINF_HI(void) 
{
	char* c;
	float f;
	c = (char*)&f;
	c[0] = 0;	c[1] = 0;
	c[2] = 0;	c[3] = 0xFF;
	return f;
}




float Trigo_returnINF_NAN(void) 
{
	char* c;
	float f;
	c = (char*)&f;
	c[0] = 0xFF;		c[1] = 0xFF;
	c[2] = 0xFF;	c[3] = 0xFF;
	return f;
}



/*------------------------------------------------------------------------------
						Programme Horner
  ------------------------------------------------------------------------------
Calcul d'un polynome de degrs n												*/

float Trigo_horner(float x, int rec, float *tab)
{
	float res=*(tab++);
	int i;
	
	for(i=0;i<rec-1;i++)res=*(tab++) + res * x;
	
	return(res);
	
}

// Fonction valeur absolue entire
int Trigo_abs(int x)
{
	if(x<0)return(-x);
	return(x);
}



// Fonction valeur absolue flottant
float Trigo_fabs(float x)
{
	if(x<0)return(-x);
	return(x);

}




/*------------------------------------------------------------------------------
						Fonctions Trigonomtriques
  ------------------------------------------------------------------------------*/
// Fonction Sinus

float Trigo_sin(float x)
{
	char signe=0;	
		
	if (x < 0) 
	{
		x = -x;
		signe=!signe;						// Restriction  la partie positive
	}
		
	x = x-PI_2*(long)(x*INV_PI_2);			// Restriction  l'intervalle [0,2Pi[

	if(x>PI)
	{
		x -= PI;							// Restriction  l'intervalle [0,Pi[
		signe=!signe;		
	}
	
	if(x>PI_DIV2)x=PI-x;					// Restriction  l'intervalle [0,Pi/2[
	
	x=Trigo_horner(x,rec_sin,&Trigo_Mat_sin[0]);

	if(signe!=0) x = -x;
	
	return(x);
}




// Fonction Cosinus

float Trigo_cos(float x)
{
	return Trigo_sin(x + PI_DIV2);
}

// Fonction tangente

float Trigo_tan(float x)
{
	char signe=0;
	
	if (x < 0) 
	{
		x = -x;
		signe=!signe;					// Restriction  la partie positive
	}
	
	x = x-PI*(long)(x*INV_PI);			// Restriction  l'intervalle [0,Pi/2[
	
	x=Trigo_horner(x,rec_sin,&Trigo_Mat_sin[0])/Trigo_horner(PI_DIV2-x,rec_sin,&Trigo_Mat_sin[0]);
	
	if(signe!=0) x = -x;
	
	return(x);
}




/*------------------------------------------------------------------------------
						Fonctions Trigonomtriques Inverses
  ------------------------------------------------------------------------------*/


// Fonction Arc tangente
float Trigo_atan(float x)
{
	char signe=0;
	char sup=0;
	
	if (x < 0) 
	{
		x = -x;
		signe=!signe;					// Restriction  la partie positive
	}
	
	if (x > 1) 
	{
		sup=1;							
		x=1/x;							// Restriction  l'intervalle [0,1[
	}
	
	x=Trigo_horner(x,rec_atn,&Trigo_Mat_atn[0]);
	
	if(sup==1)x=PI_DIV2-x;

	if(signe!=0)x=-x;
	
	return(x);
}



// Fonction Arc tangente 2, calcul l'angle form par le vecteur x,y
float Trigo_atan2(float y,float x)
{
        if (y==0 && x==0) return Trigo_returnINF_NAN();			// Retourne Not A Number si x=0 et y=0
        if (x==0 && y>0)  return(PI_DIV2);						// Retourne PI/2 (cas particulier)
        if (x==0 && y<0)  return(-PI_DIV2);						// Retourne -PI/2 (cas particulier)

        if (x<0 && y<0)  return (-PI+Trigo_atan(y/x));			// Cadran Bas Gauche : on ajoute -PI
        if (x<0 && y>=0) return (PI+Trigo_atan(y/x));			// Cadran Bas Droit  : on ajoute PI

        return (Trigo_atan(y/x));								// Cadran droit : on retourne la valeur brute
}
	


